@@ -10,8 +10,8 @@
 #ifndef OKULAR_VIEW_H
 #define OKULAR_VIEW_H
 
-#include "okular_export.h"
-
+#include "okularcore_export.h"
+#include <QObject>
 class QString;
 class QVariant;
 
@@ -29,7 +29,7 @@ class ViewPrivate;
  *
  * @since 0.7 (KDE 4.1)
  */
-class OKULAR_EXPORT View
+class OKULARCORE_EXPORT View
 {
     /// @cond PRIVATE
     friend class Document;
