@@ -31,11 +31,11 @@ class BookmarkList : public QWidget, public Okular::DocumentObserver
     Q_OBJECT
 
     public:
-        explicit BookmarkList( Okular::Document *document, QWidget *parent = Q_NULLPTR );
+        explicit BookmarkList( Okular::Document *document, QWidget *parent = nullptr );
         ~BookmarkList();
 
         // inherited from DocumentObserver
-        void notifySetup( const QVector< Okular::Page * > & pages, int setupFlags ) Q_DECL_OVERRIDE;
+        void notifySetup( const QVector< Okular::Page * > & pages, int setupFlags ) override;
 
     private Q_SLOTS:
         void slotFilterBookmarks( bool );
